module ActiveRecord
  module PGCollation
    register :schema_dumper do
      require "active_record/connection_adapters/postgresql/schema_dumper"
      ActiveRecord::ConnectionAdapters::PostgreSQL::SchemaDumper.prepend(SchemaDumper)
    end

    module SchemaDumper
      private

      def extensions(stream)
        super
        collations(stream)
      end

      def collations(stream)
        collations = @connection.collations

        if collations.any?
          stream.puts(%Q{  # These are collations that must be created before they can be used in the schema definition})
          collations.each do |collation|
            name = collation["name"]
            deterministic = collation["deterministic"]
            options = collation.except("name", "deterministic").map {|k,v| %Q{#{k}: "#{v}"}}.join(", ")
            stream.puts(%Q{  create_collation "#{name}", #{options}, deterministic: #{deterministic}})
          end
          stream.puts
        end
      end
    end
  end
end

