module ActiveRecord
  module PGCollation
    register :postgresql_adapter do
      require "active_record/connection_adapters/postgresql_adapter"
      ActiveRecord::ConnectionAdapters::PostgreSQLAdapter.include(::ActiveRecord::PGCollation::PostgreSQLAdapter)
    end

    module PostgreSQLAdapter
      # Helper method used by the monkeypatch internals. Provides an array of collations as they exist currently.
      def collations
        provider_mapping = {"i" => "icu", "c" => "libc", "d" => "libc"}
        result = ActiveRecord::Base.connection.exec_query(<<-SQL.strip_heredoc, "SCHEMA")
          SELECT c.collname AS name, c.collcollate AS lc_collate, c.collctype AS lc_ctype, c.collprovider AS provider, c.collisdeterministic AS deterministic
          FROM pg_collation c
          JOIN pg_catalog.pg_namespace n ON n.oid = c.collnamespace
          WHERE n.nspname = ANY (current_schemas(false))
        SQL
        result.to_a.sort_by {|h| h["name"]}.each {|h| h["provider"] = provider_mapping[h["provider"]]}
      end
    end
  end
end
